//----------------------------------------------------------------------------------------
// error_computation.H: Contains declarations of functions computing norm of error
// or change for unknowns
//----------------------------------------------------------------------------------------

#ifndef ERROR_COMPUTATION_H
#define ERROR_COMPUTATION_H

#include "general.H"

// Global error computation in 3D
// Parameters: t - time; func - analytical solution; norm_type - norm choice index;
//             error - error in the given norm
void global_error(real t, void func(real, const real[d_space], real[]), int norm_type,
                  real error[d_var_3d]);

// Global change between two consecutive time steps in 3D
// Parameters: dt - time step; norm_type - norm choice index; change - change in the given norm
void global_change(real dt, int norm_type, real change[]);

// Error in surface elevation computation
// Parameters: t - time; func - analytical solution; norm_type - norm choice index;
// Return value: error in the given norm
real elevation_error(real t, void func(real, const real[d_space_2d], real[]), int norm_type);

// Change in surface elevation between two consecutive time steps in 3D
// Parameters: dt - time step; norm_type - norm choice index;
// Return value: change in the given norm
real elevation_change(real dt, int norm_type);

#ifdef MASS_CONSERVATION_KEY
// Tracer mass conservation error
// Parameters: t - time
// Return value: error in mass balance
long_real tracer_mass_error(real t);
    
#endif
#endif
