//----------------------------------------------------------------------------------------
// test.H: Headers for analytical solution for test problems
//----------------------------------------------------------------------------------------

#ifndef TEST_H
#define TEST_H

#include "general.H"
#include "initial_data.H"

// True solution for the test problem
void solution_analytic(real, const real[d_space], real[d_var_3d]);

// Gradient of the true solution
void solution_analytic_grad(real, const real[d_space], real[d_var_3d][d_space]);

// Laplacian of the true solution
void solution_analytic_laplacian(real, const real[d_space], real[d_var_3d_dt][d_space]);

// Time derivative of the true solution for the test problem
void solution_analytic_dt(real, const real[d_space], real[d_var_3d_dt]);

// Forcing function for the test problem
void solution_analytic_forcing(real, const real[d_space], real[d_var_3d_dt]);

// Analytic solution for the elevation
void solution_analytic_elevation(real, const real[2], real[]);

// Analytic solution for the elevation gradient
void solution_analytic_elevation_grad(real, const real[2], real[2]);

// Time derivative of the true solution for the elevation
void solution_analytic_elevation_dt(real, const real[d_space], real[]);

// Forcing function for the elevation
void solution_analytic_elevation_forcing(real, const real[d_space], real[]);

#ifdef BAROCLINIC_KEY
// Density forcing
void density_forcing(real, real, const real[], const real[], real[]);
#endif

// Vertical eddy viscosity/diffusion computation
void diffusion_vert_coef(real t, const real coord[d_space], const real var[d_var], 
                         const real grad[d_var][d_space], real diff_coef[d_var][d_space], 
                         real diff_coef_grad[d_var][d_space], real rhs[d_var]);
#endif
